@@ -248,17 +248,53 @@ keygripstr_from_pk_file (app_t app, int fid, char *r_gripstr)
 
 
 /* TCOS responds to a verify with empty data (i.e. without the Lc
- * byte) with the status of the PIN.  PWID is the PIN ID, If SIGG is
- * true, the application is switched into SigG mode.  Returns:
- * ISO7816_VERIFY_* codes or non-negative number of verification
- * attempts left.  */
+   byte) with the status of the PIN.  PWID is the PIN ID, If SIGG is
+   true, the application is switched into SigG mode.
+   Returns:
+            -1 = Error retrieving the data,
+            -2 = No such PIN,
+            -3 = PIN blocked,
+            -4 = NullPIN activ,
+        n >= 0 = Number of verification attempts left.  */
 static int
 get_chv_status (app_t app, int sigg, int pwid)
 {
+  unsigned char *result = NULL;
+  size_t resultlen;
+  char command[4];
+  int rc;
+
   if (switch_application (app, sigg))
     return sigg? -2 : -1; /* No such PIN / General error.  */
 
-  return iso7816_verify_status (app_get_slot (app), pwid);
+  command[0] = 0x00;
+  command[1] = 0x20;
+  command[2] = 0x00;
+  command[3] = pwid;
+
+  if (apdu_send_direct (app->slot, 0, (unsigned char *)command,
+                        4, 0, NULL, &result, &resultlen))
+    rc = -1; /* Error. */
+  else if (resultlen < 2)
+    rc = -1; /* Error. */
+  else
+    {
+      unsigned int sw = buf16_to_uint (result+resultlen-2);
+
+      if (sw == 0x6a88)
+        rc = -2; /* No such PIN.  */
+      else if (sw == 0x6983)
+        rc = -3; /* PIN is blocked.  */
+      else if (sw == 0x6985)
+        rc = -4; /* NullPIN is activ.  */
+      else if ((sw & 0xfff0) == 0x63C0)
+        rc = (sw & 0x000f); /* PIN has N tries left.  */
+      else
+        rc = -1; /* Other error.  */
+    }
+  xfree (result);
+
+  return rc;
 }
 
 
@@ -272,10 +308,8 @@ do_getattr (app_t app, ctrl_t ctrl, const char *name)
     int special;
   } table[] = {
     { "$AUTHKEYID",   1 },
-    { "$ENCRKEYID",   2 },
-    { "$SIGNKEYID",   3 },
-    { "NKS-VERSION",  4 },
-    { "CHV-STATUS",   5 },
+    { "NKS-VERSION",  2 },
+    { "CHV-STATUS",   3 },
     { NULL, 0 }
   };
   gpg_error_t err = 0;
@@ -305,27 +339,13 @@ do_getattr (app_t app, ctrl_t ctrl, const char *name)
       }
       break;
 
-    case 2: /* $ENCRKEYID */
-      {
-        char const tmp[] = "NKS-NKS3.45B1";
-        send_status_info (ctrl, table[idx].name, tmp, strlen (tmp), NULL, 0);
-      }
-      break;
-
-    case 3: /* $SIGNKEYID */
-      {
-        char const tmp[] = "NKS-NKS3.4531";
-        send_status_info (ctrl, table[idx].name, tmp, strlen (tmp), NULL, 0);
-      }
-      break;
-
-    case 4: /* NKS-VERSION */
+    case 2: /* NKS-VERSION */
       snprintf (buffer, sizeof buffer, "%d", app->app_local->nks_version);
       send_status_info (ctrl, table[idx].name,
                         buffer, strlen (buffer), NULL, 0);
       break;
 
-    case 5: /* CHV-STATUS */
+    case 3: /* CHV-STATUS */
       {
         /* Returns: PW1.CH PW2.CH PW1.CH.SIG PW2.CH.SIG That are the
            two global passwords followed by the two SigG passwords.
@@ -365,7 +385,6 @@ do_learn_status_core (app_t app, ctrl_t ctrl, unsigned int flags, int is_sigg)
   char ct_buf[100], id_buf[100];
   int i;
   const char *tag;
-  const char *usage;
 
   if (is_sigg)
     tag = "SIGG";
@@ -415,19 +434,9 @@ do_learn_status_core (app_t app, ctrl_t ctrl, unsigned int flags, int is_sigg)
             {
               snprintf (id_buf, sizeof id_buf, "NKS-%s.%04X",
                         tag, filelist[i].fid);
-              if (filelist[i].issignkey && filelist[i].isenckey)
-                usage = "sae";
-              else if (filelist[i].issignkey)
-                usage = "sa";
-              else if (filelist[i].isenckey)
-                usage = "e";
-              else
-                usage = "";
-
               send_status_info (ctrl, "KEYPAIRINFO",
                                 gripstr, 40,
                                 id_buf, strlen (id_buf),
-                                usage, strlen (usage),
                                 NULL, (size_t)0);
             }
         }
